#include <errno.h>
#include <libmp/util.h>
#include <spawn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>
#include <wait.h>

#include "log.h"
#include "util.h"

char **env = NULL;

void set_env(char **envp) {
  env = envp;
}

bool startswith(char *str, char *sub) {
  size_t strl = strlen(str);
  size_t subl = strlen(sub);

  if (subl > strl)
    return false;

  return strncmp(sub, str, subl) == 0;
}

void size_to_human(char *buf, long size) {
  char *suffix[] = {"B", "KB", "MB", "GB", "TB"};
  char  length   = sizeof(suffix) / sizeof(suffix[0]);

  int    i       = 0;
  double cursize = size;

  if (size > 1024) {
    for (i = 0; (size / 1024) > 0 && i < length - 1; i++, size /= 1024)
      cursize = size / 1024.0;
  }

  sprintf(buf, "%.02lf%s", cursize, suffix[i]);
}

char *exists_in_path(char *file) {
  if (NULL == file)
    return NULL;

  char  *path = NULL, *part = NULL, *save = NULL;
  size_t filesz = strlen(file);

  if ((path = getenv("PATH")) == NULL)
    path = "/bin";

  if ((part = strtok_r(path, ":", &save)) == NULL)
    return false;

  do {
    char fp[strlen(part) + filesz + 1];
    join(fp, part, file);

    if (exists(fp))
      return strdup(fp);
  } while ((part = strtok_r(NULL, ":", &save)) != NULL);

  return NULL;
}

void editor_open(char *file) {
  if (NULL == file)
    return;

  char *editors[] = {getenv("EDITOR"), "nano", "vi", "vim"};
  int   status = 0, i = 0;
  char *editor = NULL;
  pid_t pid    = 0;

  for (; i < sizeof(editors) / sizeof(char *); i++) {
    if ((editor = exists_in_path(editors[i])) != NULL)
      break;
  }

  if (NULL == editor) {
    error(_("No available editor to open " FG_BOLD "%s" FG_RESET), file);
    return;
  }

  char *argv[] = {editor, file, NULL};

  if (posix_spawnp(&pid, argv[0], NULL, NULL, argv, env) != 0) {
    error(_("Failed to run the editor: %s"), strerror(errno));
    goto end;
  }

  if (waitpid(pid, &status, 0) < 0) {
    error(_("Failed to open the editor: %s"), strerror(errno));
    goto end;
  }

end:
  free(editor);
  return;
}

bool can_run_install(lm_ctx_t *ctx) {
  return eq(ctx->root, "/");
}
